/* Zdrojov text: 26D_ITriangleSSS.jsh
############################# Pli luouk k pl belsk .*/

/***************************************************************************
 * Pravohl trojhelnk definovan dlkami stran podle vty SSS.
 */
public record RightTriangleSSS(double a, double b, double c, int ID)
implements ITriangleSSS
{
    private static final int MI = Integer.MAX_VALUE;
    private static int count = 0;

    /***********************************************************************
     * Vytvo zznam o pravohlm trojhelnku zadanm dlkami svch stran.
     * Kontroluje platnost Pythagorovy vty.
     */
    public RightTriangleSSS
    {
        if (ID != MI) {
            throw new IllegalArgumentException(
                "\nPouvn konstruktoru zadvajcho vechny dlky stran"
              + " je zakzno!");
        }
        this.ID = ++count;
    }

    /***********************************************************************
     * Vytvo zznam o pravohlm trojhelnku zadanm dlkou odvsen.
     * Dlku pepony dopote podle Pythagorovy vty.
     */
    public RightTriangleSSS(double a, double b)
    {
        this(a, b, Math.sqrt(a*a + b*b), MI);
    }

    /***********************************************************************
     * Vytvo zznam o rovnoramennm pravohlm trojhelnku zadanm dlkou
     * sv pepony. Dlku odvsen dopote podle Pythagorovy vty.
     */
    public RightTriangleSSS(double c)
    {
        this(Math.sqrt(c*c / 2), Math.sqrt(c*c / 2), c, MI);
    }

    /***********************************************************************
     * Spote hly u pepony pravohlho trojhelnka a vrt pole
     * s jejich velikost zadanou ve stupnch.
     */
    public double[] alphaBeta()
    {
        double alpha = Math.toDegrees(Math.acos(a()/c()));
        double beta  = Math.toDegrees(Math.acos(b()/c()));
        return new double[] { alpha, beta };
    }

    /***********************************************************************
     * Pi porovnvn instanc kontroluje pouze dlky stran;
     * hodnota ID ani konkrtn typ objektu jej nezajmaj.
     */
    public boolean equals(Object o)
    {
        var result = false;
        if (o instanceof ITriangleSSS t) {
            result = (a == t.a())  &&  (b == t.b())  &&  (c==t.c());
        }
        return result;
    }

    /***********************************************************************
     * Generuje hekd zvisejc pouze na dlkch stran.
     */
    public int hashCode()
    {
        DoubleToIntFunction h = (double d) ->
                           (int)( Double.doubleToLongBits(d) ^
                                 (Double.doubleToLongBits(d) >>> 32));
        int hash = ((97 + h.applyAsInt(a)) *97
                        + h.applyAsInt(b) )*97 + h.applyAsInt(c);
        return hash;
    }

}
